#include <config.h>
#include <gnome.h>
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

#include "defs.h"
#include "file.h"

GString *extract_hostname(GString *name) {
	gchar *tmp;
	gchar *pos;
	GString *ret;
	
	tmp = (gchar*) g_malloc(sizeof(gchar)*(name->len+1));
	strcpy(tmp, name->str);
	pos = index(tmp,':');
	if(pos) {
		pos[0] = 0;
		ret = g_string_new(tmp);
		free(tmp);
		return(ret);
	} else return(NULL);
}

guint extract_port(GString *name) {
	gchar *tmp;
	gchar *pos;
	guint ret;
	
	tmp = (gchar*) g_malloc(sizeof(gchar)*(name->len+1));
	strcpy(tmp, name->str);
	pos = index(tmp,':');
	if(pos) {
		ret = atoi(++pos);
		free(tmp);
		return(ret);
	} else return(0);
}


/* Put the list of buttons into my_config.
 * The list of buttons is of the form
 * button1[:label1][,button2[:label2]|,...]
 */

void parse_buttons(GString* buttons, OPTIONS *my_config) {
	char *begin, *end;
	char separator;
	int i;
	gint value;
	gchar *tmp;
	
	tmp = (gchar*)g_malloc(sizeof(char)*MAX_STRING_LENGTH);
	
	begin = buttons->str;
	if(begin[0]) {
		my_config->value = g_array_new(TRUE, TRUE, sizeof(gint));
		my_config->label = g_ptr_array_new();
		my_config->number = g_ptr_array_new();
	}
	
	end = begin;
	i = 0;
	while(end[0] != 0) {
		while((end[0] != 0) && (end[0] != ':') && (end[0] != ',')) end++;
		separator = end[0];
		end[0] = 0;
		g_ptr_array_add(my_config->label, g_string_new(begin));
		snprintf(tmp, MAX_STRING_LENGTH, "%d", i);
		g_ptr_array_add(my_config->number, g_string_new(tmp));
		end[0] = separator;
		
		if(separator) {
			begin = end+1;
			end = begin;
			if(separator == ':') {
				while((end[0] != 0) && (end[0] != ':') && (end[0] != ',')) end++;
				separator = end[0];
				end[0] = 0;
				value = atoi(begin);
				g_array_append_val(my_config->value, value);
				end[0] = separator;
				if(separator) {
					begin = end+1;
					end = begin;
				}
			} else {
				value = 0;
				g_array_append_val(my_config->value, value);
			}
		}
		i++;
	}
	free(tmp);
}

void usage(poptContext optCon, int exitcode, char *error) {
	poptPrintUsage(optCon, stderr, 0);
	if (error) g_error(error);
	exit(exitcode);
}

void parse_command_line(poptContext optCon, OPTIONS *my_options) {
	gchar *message;
	gint value;
	gint i;
	gint flags;
	GString *label;
	GString *server_name;
	guint server_port;
	struct sockaddr_in mysockaddr;

#ifndef socklen_t
typedef size_t socklen_t;
#endif

	socklen_t lg;

	message = poptGetArg(optCon);
	if(((message == NULL) || !(poptPeekArg(optCon) == NULL))
			&& (my_options->filename == NULL)
			&& (my_options->message == NULL))
		usage(optCon, 1, _("Specify a message to display\n"));

	if(my_options->message == NULL) {

		if(my_options->filename == NULL) {
			/* No file name */
			my_options->message = g_string_new(message);
		} else {
			/* filename was given on the command line */

			if(!strcmp((my_options->filename)->str,"-")) {

				/* the file name is stdin */
				my_options->fh = dup(STDIN_FILENO);
				flags = fcntl(my_options->fh, F_GETFL);
				fcntl(my_options->fh, F_SETFL, flags|O_NONBLOCK);
				my_options->message = read_fh(my_options->fh);
				
			} else if(!strncmp((my_options->filename)->str,"socket://",9)) {

				my_options->filename = g_string_erase(my_options->filename, 0, 9);
				server_name = extract_hostname(my_options->filename);
				server_port = extract_port(my_options->filename);
				if(server_name && server_port) {
					my_options->fh = connect_to_socket(server_name, server_port);
					if(my_options->fh == -1) {
						my_options->message = g_string_new(_("Unable to connect to the specified host."));
					} else {
						if(my_options->follow) {
							flags = fcntl(my_options->fh, F_GETFL);
							fcntl(my_options->fh, F_SETFL, flags|O_NONBLOCK);
						}
						my_options->message = read_fh(my_options->fh);
						if(!my_options->follow) close(my_options->fh);
					}
				} else {
					my_options->message = g_string_new(_("Unknown format for the specified host."));
				}
				
			} else if(!strncmp((my_options->filename)->str,"server://",9)) {
				my_options->filename = g_string_erase(my_options->filename, 0, 9);
				server_port = atoi(my_options->filename->str);
				if(server_port) {
					my_options->sock = make_server(server_port);
					if(my_options->sock == -1) my_options->message = g_string_new(_("Unable to create local server"));
					else {
						lg = sizeof(struct sockaddr_in);
						my_options->fh = accept(my_options->sock, &mysockaddr, &lg);
						if(my_options->follow) {
							flags = fcntl(my_options->fh, F_GETFL);
							fcntl(my_options->fh, F_SETFL, flags|O_NONBLOCK);
						}
						if(write_fh(my_options->fh, _("Type a message... (Ctrl-D to quit)\n"))) my_options->message = g_string_new(_("Could not write to the specified host."));
/*						else my_options->message = read_fh(my_options->fh);*/
					}
				} else {
					my_options->message = g_string_new(_("Unknown format for the specified host."));
				}
				
			} else {
				/* the file name is not stdin */
				if(my_options->follow) {
					/* follow option is YES */
					my_options->fh = open((my_options->filename)->str,(O_RDONLY|O_NONBLOCK|O_EXCL));
					if(my_options->fh < 0) {
						my_options->message = g_string_new("Could not open ");
						g_string_append(my_options->message, (my_options->filename)->str);
					} else
						my_options->message = read_fh(my_options->fh);
				} else {
					/* follow option is NO */
					my_options->message = read_file(my_options->filename);
				}
			}
		}
	}
    	    if(!my_options->label) {
    		    my_options->value = g_array_new(TRUE, TRUE, sizeof(gint));
    		    my_options->label = g_ptr_array_new();
    		    my_options->number = g_ptr_array_new();
    		    g_ptr_array_add(my_options->label, g_string_new(_("Okay")));
    		    g_ptr_array_add(my_options->number, g_string_new("0"));
    		    value = 0;
    		    g_array_append_val(my_options->value, value);

    	    }

    	    if(my_options->default_label) {
    		    i = 0;
    		    while(i < (my_options->number)->len) {
    			    label = g_ptr_array_index(my_options->label,i);
    			    if(strcmp(label->str, (my_options->default_label)->str) == 0) {
    				    my_options->clicked_button = i;
    				    i = (my_options->number)->len;
    			    }
    			    i++;
    		    }
    	    }

}
