#include <config.h>
#ifdef HAVE_POPT
#include <popt.h>
#include <glib.h>
#include <gettext.h>
#include <libintl.h>
#define _(String) gettext (String)
#    ifdef gettext_noop
#        define N_(String) gettext_noop (String)
#    else
#        define N_(String) (String)
#    endif
#else
#include <gnome.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <signal.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>

#include "f2sd.h"

#ifndef socklen_t
typedef size_t socklen_t;
#endif


GString *read_file(GString *filename) {
	gint fh;
	GString *message;
	gint len, total_len;
	gchar tmp[INITIAL_PIPE_BUFFER_LENGTH];
	
	fh = open(filename->str,O_RDONLY|O_EXCL);
	if(fh < 0) {
		message = NULL;
	} else {
		message = g_string_sized_new(INITIAL_PIPE_BUFFER_LENGTH);
		total_len = 0;
		while((len=read(fh, tmp, INITIAL_PIPE_BUFFER_LENGTH-1)) > 0) {
			total_len += len;
			tmp[len]=0;
			g_string_append(message, tmp);
		}
		if(!total_len) {
			g_string_free(message, TRUE);
			message = NULL;
		}
		close(fh);
	}
	
	return(message);
}

int make_server(uint16_t port) {

	char *server_name;
	int sock,res;
	struct sockaddr_in *myserver;
	struct hostent *machine_info;

/* What is the name of this computer ? */
	if(!(server_name=(char*)g_malloc(sizeof(char)*128))) {
		g_error("Hostname g_malloc ");
		exit(-1);
	}
	
	if(gethostname(server_name,128)) {
		g_error("Hostname gethostname ");
		exit(-1);	
	}

/* Open the server socket */

	sock = socket(AF_INET, SOCK_STREAM,6);
	
	if(sock == -1) {
		g_error("Socket ");
		exit(1);
	}
	
/* Get information about the machine */

	if((machine_info = gethostbyname(server_name)) == NULL) {
		g_error("gethostbyname ");
		exit(1);
	}
	
/* Creation of the server */
	if((myserver = (struct sockaddr_in *)g_malloc(sizeof(struct sockaddr_in)))==NULL) {
		g_error("g_malloc on server creation ");
		exit(1);
	}
	
	bzero(myserver, sizeof(struct sockaddr_in));
	
	myserver->sin_family = machine_info->h_addrtype;
	myserver->sin_port = htons(port);
	myserver->sin_addr.s_addr = INADDR_ANY;
	res = bind(sock, myserver, sizeof(struct sockaddr_in));
	
	if(res == -1) {
		g_error("bind ");
		exit(1);
	}
	
	res = listen(sock,10);

	if(res == -1) {
		g_error("listen ");
		exit(1);
	}
	return(sock);
}


static char *filename=NULL;
static char *myport=NULL;

struct poptOption command_line[] = {
	{ "file", 'f', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &filename, F_FILE, 
			N_("File to send"), "filename" },
	{ "port", 'p', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &myport, F_PORT, 
			N_("Port to listen to"), "port" },
	POPT_AUTOHELP
	{ NULL, 0, 0, NULL, 0 }
};

void zombie(int sig){
	signal(SIGCHLD, zombie);
	while (wait3(NULL, WNOHANG, NULL) > 0);
}


#ifdef HAVE_POPT
int main(int argc, const char** argv) {
#else
int main(int argc, char** argv) {
#endif
	F2SS	myconfig;
	int sock, sock2;
	struct sockaddr_in mysockaddr;
	socklen_t lg;
	GString *message = NULL;
	poptContext optCon;
	gchar c;
	
	bindtextdomain (PACKAGE,GNOMELOCALEDIR);
	textdomain(PACKAGE);
#ifndef HAVE_POPT
	gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv, command_line, 0, &optCon);
#endif
	optCon = poptGetContext(NULL, argc, argv, command_line, 0);

/* Processing of arguments */

	if (argc < 4) {
		poptPrintUsage(optCon, stderr, 0);
		exit(1);
	}

	while ((c = poptGetNextOpt(optCon)) >= 0) {
		switch (c) {
			case F_FILE:
				myconfig.file = g_string_new(filename);
				message = read_file(myconfig.file);
				break;
			case F_PORT:
				myconfig.port = atoi(myport);
				if(myconfig.port <= 0) g_error(_("Wrong port number"));
				break;
		}
	}

	if(message == NULL) g_error(_("File does not exist or is not readable\n"));
	
	if (c < -1) {
	/* an error occurred during option processing */
	fprintf(stderr, "%s: %s\n",
		poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
		poptStrerror(c));
		return(-1);
	}
	
/* Create and run the server */
	sock = make_server(myconfig.port);
	signal(SIGCHLD, zombie);
	while(1) {
		lg = sizeof(struct sockaddr_in);
		sock2 = accept(sock, &mysockaddr, &lg);
		if(sock2 >=0) {
			switch(fork()) {
				case 0:
					close(sock);
					write(sock2, message->str, message->len);
					close(sock2);
					exit(0);
				default:
					close(sock2);
					break;
			}
		}
	}
	exit(0);
}
