#include <config.h>
#include <gnome.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "defs.h"

/* Read through the pipe and return the result as a GString */
GString *read_fh(gint fh) {
	GString *message;
	gchar tmp[INITIAL_PIPE_BUFFER_LENGTH];
	gint len, total_len;
	
	message = g_string_sized_new(INITIAL_PIPE_BUFFER_LENGTH);
	total_len = 0;
	while(((len=read(fh, tmp, INITIAL_PIPE_BUFFER_LENGTH-2)) > 0) ){
		tmp[len]=0;
		if(tmp[0] == 4) {
			tmp[1] = 0;
			len = 1;
		}
		total_len += len;
		g_string_append(message, tmp);
	}

	if(total_len) {
		return(message);
	} else {
		g_string_free(message, TRUE);
		return(NULL);
	}
}

int write_fh(gint fh, char *string) {
	gint l;
	l = strlen(string);
	if(l) {
		if(l != write(fh, string, l)) return(-1);
	}
	return(0);
}

/* Read a file and return the result as a GString */
GString *read_file(GString *filename) {
	gint fh;
	GString *message;
	
	fh = open(filename->str,O_RDONLY|O_NONBLOCK|O_EXCL);
	if(fh < 0) {
		message = g_string_new(_("Could not open "));
		g_string_append(message, filename->str);
	} else
		message = read_fh(fh);
	close(fh);
	
	return(message);
}


int make_server(uint16_t port) {

	char *server_name;
	int sock,res;
	struct sockaddr_in *myserver;
	struct hostent *machine_info;

/* What is the name of this computer ? */
	if(!(server_name=(char*)g_malloc(sizeof(char)*128))) {
		g_error("Hostname g_malloc ");
		exit(-1);
	}
	
	if(gethostname(server_name,128)) {
		g_error("Hostname gethostname ");
		exit(-1);	
	}

/* Open the server socket */

	sock = socket(AF_INET, SOCK_STREAM,6);
	
	if(sock == -1) {
		g_error("Socket ");
		exit(1);
	}
	
/* Get information about the machine */

	if((machine_info = gethostbyname(server_name)) == NULL) {
		g_error("gethostbyname ");
		exit(1);
	}
	
/* Creation of the server */
	if((myserver = (struct sockaddr_in *)g_malloc(sizeof(struct sockaddr_in)))==NULL) {
		g_error("g_malloc on server creation ");
		exit(1);
	}
	
	bzero(myserver, sizeof(struct sockaddr_in));
	
	myserver->sin_family = machine_info->h_addrtype;
	myserver->sin_port = htons(port);
	myserver->sin_addr.s_addr = INADDR_ANY;
	res = bind(sock, myserver, sizeof(struct sockaddr_in));
	
	if(res == -1) {
		g_error("bind ");
		exit(1);
	}
	
	res = listen(sock,10);

	if(res == -1) {
		g_error("listen ");
		exit(1);
	}
	return(sock);
}

int connect_to_socket(GString *ip, uint16_t port) {
	struct hostent server_address, *address;
	struct sockaddr_in *client_sockaddr_in;
	int sock;
	
	if((address = gethostbyname(ip->str)) == NULL) {
		perror("gethostbyname ");
		exit(1);
	}
	memcpy(&server_address,address,sizeof(struct hostent));
	
/*	printf("Local IP address : %d.%d.%d.%d\n",(unsigned char)server_address.h_addr[0], (unsigned char)server_address.h_addr[1], (unsigned char)server_address.h_addr[2], (unsigned char)server_address.h_addr[3]);
 */

/* Open the server socket */

	sock = socket(AF_INET, SOCK_STREAM,6);
	if(sock == -1) {
		perror("Socket ");
		exit(1);
	}
/* Get infos on the server */
	if((client_sockaddr_in = (struct sockaddr_in *)g_malloc(sizeof(struct sockaddr_in)))==NULL) {
		perror("g_malloc on server creation ");
		exit(1);
	}
	
	client_sockaddr_in->sin_family = server_address.h_addrtype;
	client_sockaddr_in->sin_port = htons(port);
	memcpy((char *)&(client_sockaddr_in->sin_addr.s_addr),server_address.h_addr,server_address.h_length);
	bzero((char *)&(client_sockaddr_in->sin_zero),sizeof (struct sockaddr) -
			   				sizeof (uint16_t) -
			   				sizeof (struct in_addr));
	
/* Connect to the server as a client */

	if(connect(sock,client_sockaddr_in, sizeof(struct sockaddr_in))) {
		sock = -1;
	}
	return(sock);
}
