#include <config.h>
#include <gnome.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "defs.h"
#include "file.h"
#include "gui.h"

#ifndef socklen_t
typedef size_t socklen_t;
#endif

void hello (GtkWidget * widget, OPTIONS *my_options) {
	gchar *name;
	GString *number;
	gint i;
	
	name = gtk_widget_get_name(widget);
	i = (my_options->number)->len - 1;
	while(i>= 0) {
		number = g_ptr_array_index(my_options->number, i);
		if(strcmp(number->str, name) == 0) {
			my_options->clicked_button = atoi(number->str);
			i = 0;
		}
		i--;
	}
	
}

/* Called when the user quits */
void destroy (GtkWidget * widget, OPTIONS *my_options) {
	gtk_main_quit ();
}

void make_gui(OPTIONS *my_options) {

	GtkWidget	*window;
	GtkWidget	*hbox;
	GtkWidget	*button;
	GtkWidget	*hscrollbar=NULL;
	GtkWidget	*vscrollbar=NULL;
	gint		i;
	gint		i2;
	GString		*button_string;
	GString		*button_number;
	gint		win_x, win_y;
	gint		win_w, win_h;
	GtkStyle *textstyle = NULL;
	gint		max_pixel_length, pixel_length;
	char		*pos;
	char		*len;
	GtkRequisition	vscd;
	
/* create a new window */
	window = gnome_app_new (PACKAGE, "gmessage+");
	gtk_window_set_position(GTK_WINDOW(window),my_options->win_position);
	gtk_window_set_title(GTK_WINDOW(window),(my_options->wintitle)->str);
	
/* Some position stuff.*/
	if(my_options->geometry) {
		if(gnome_parse_geometry((my_options->geometry)->str,&win_x, &win_y, &win_w, &win_h) == FALSE) {
			win_x = 50;
			win_y = 50;
			win_w = 400;
			win_h = 200;
			g_warning("Incorrect geometry parameters\n");
		}
		gtk_window_set_default_size(GTK_WINDOW(window), win_w, win_h);
		gtk_widget_set_uposition(window, win_x, win_y);
	}

	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);

	gtk_signal_connect (GTK_OBJECT (window), "destroy",
		GTK_SIGNAL_FUNC (destroy), my_options);


/* Create the content of the window */

/* Create the table */
	if(my_options->nobuttons == TRUE) {
		my_options->msgtable = gtk_table_new(2,2,FALSE);
	} else {
		my_options->msgtable = gtk_table_new(2,3,FALSE);
	}
	gtk_table_set_row_spacing (GTK_TABLE (my_options->msgtable), 0, 2);
	gtk_table_set_col_spacing (GTK_TABLE (my_options->msgtable), 0, 2);

	gnome_app_set_contents(GNOME_APP(window), my_options->msgtable);


/* Message (gtk_text) */
	my_options->msgbox = gtk_text_new (NULL, NULL);
	gtk_table_attach (GTK_TABLE (my_options->msgtable), my_options->msgbox, 0, 1, 0, 1,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

/* Add a vertical scrollbar to the GtkText widget */
	if(my_options->v_scrollbar == TRUE) {
		vscrollbar = gtk_vscrollbar_new (GTK_TEXT (my_options->msgbox)->vadj);
		gtk_table_attach (GTK_TABLE (my_options->msgtable), vscrollbar, 1, 2, 0, 1,
                          GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
	}

/* Add an horizontall scrollbar to the GtkText widget */
	if(my_options->h_scrollbar == TRUE) {
		hscrollbar = gtk_hscrollbar_new (GTK_TEXT (my_options->msgbox)->hadj);
		gtk_table_attach (GTK_TABLE (my_options->msgtable), hscrollbar, 0, 1, 1, 2,
                          GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
	}
	

	if(my_options->nobuttons == FALSE) {
/* HBOX (for the gtk_buttons) */
		hbox = gtk_hbox_new(FALSE, 0);
		gtk_table_attach (GTK_TABLE (my_options->msgtable), hbox, 0, 2, 2, 3,
			GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);

/* Buttons inside the hbox */	
		for(i=0; i<my_options->label->len; i++) {
			if(my_options->align_buttons == SB_LEFT) i2 = i;
			else i2 = my_options->label->len-i-1;
			
			button_string = g_ptr_array_index(my_options->label, i2);
			button = gtk_button_new_with_label(button_string->str);

			button_number = g_ptr_array_index(my_options->number, i2);
			gtk_widget_set_name(button, button_number->str);

			gtk_signal_connect (GTK_OBJECT (button), "clicked",
				GTK_SIGNAL_FUNC (hello), my_options);

			gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
				GTK_SIGNAL_FUNC (gtk_widget_destroy),
				GTK_OBJECT (window));

			switch(my_options->align_buttons) {
				case SB_NO: gtk_box_pack_end(GTK_BOX(hbox), button, TRUE, TRUE, 2); break;
				case SB_LEFT: gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 2); break;
				case SB_RIGHT: gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 2); break;
				case SB_CENTER: gtk_box_pack_end(GTK_BOX(hbox), button, TRUE, FALSE, 2); break;
			}
			gtk_widget_show (button);
		}
		gtk_widget_show (hbox);
	}
	
	/* Add some text inside the messagebox if necessary */
	if(my_options->message) {
		gtk_text_insert (GTK_TEXT(my_options->msgbox), NULL, NULL, NULL, (my_options->message)->str, -1);
		if(!my_options->geometry) {
			textstyle = gtk_widget_get_style(my_options->msgbox);
			max_pixel_length = 0;
			pos = (my_options->message)->str;
			len = pos;
			while((len-(my_options->message)->str) < (my_options->message)->len) {
				len = pos;
				while((len[0] != '\n') && (len[0] != 0)) len++;
				if(len-pos) pixel_length = gdk_text_width(textstyle->font, pos, len-pos);
				else pixel_length = 0;
				if(pixel_length >= max_pixel_length) {
					max_pixel_length = pixel_length;
				}
				pos = len+1;
			}
			max_pixel_length += 16;
			if(my_options->v_scrollbar == TRUE) {
				gtk_widget_size_request(vscrollbar, &vscd);
				max_pixel_length += vscd.width;
				if(max_pixel_length > gdk_screen_width()) max_pixel_length = gdk_screen_width()-vscd.width;
			} else if(max_pixel_length > gdk_screen_width()) max_pixel_length = gdk_screen_width();
			gtk_window_set_default_size(GTK_WINDOW(window), max_pixel_length, 200);
		}
	}

/* Show everything */
	if(my_options->v_scrollbar == TRUE)
		gtk_widget_show (vscrollbar);
	if(my_options->h_scrollbar == TRUE)
		gtk_widget_show (hscrollbar);
	gtk_widget_show (my_options->msgbox);
	gtk_widget_show (my_options->msgtable);
	gtk_widget_show (window);
}

gint wait_again(gpointer my_options) {
	struct sockaddr_in mysockaddr;
	socklen_t lg;
	gint flags;
	struct timeval tv;
	int ndfs;
	fd_set rdfs, afds;

	ndfs = getdtablesize();
	FD_ZERO(&afds);
	FD_SET(((OPTIONS *)my_options)->sock,&afds);
	memcpy(&rdfs,&afds, sizeof(fd_set));
	tv.tv_sec = 0;
	tv.tv_usec = 0;
	if(select(ndfs, &rdfs, 0, 0, &tv) < 0) {
		if(errno != EINTR) {
			perror("select : ");
			exit(1);
		} else {
			gtk_timeout_add(READ_TIMEOUT, wait_again, my_options);
		}
	} else {
		if(FD_ISSET(((OPTIONS *)my_options)->sock,&rdfs)) {	
			lg = sizeof(struct sockaddr_in);
			((OPTIONS *)my_options)->fh = accept(((OPTIONS *)my_options)->sock, &mysockaddr, &lg);
			flags = fcntl(((OPTIONS *)my_options)->fh, F_GETFL);
			fcntl(((OPTIONS *)my_options)->fh, F_SETFL, flags|O_NONBLOCK);
			if(write_fh(((OPTIONS *)my_options)->fh, _("Type a message... (Ctrl-D to quit)\n"))) {
				gtk_text_insert (GTK_TEXT(((OPTIONS *)my_options)->msgbox), NULL, NULL, NULL, "Could not write to the specified host.", -1);
				gtk_widget_draw(((OPTIONS *)my_options)->msgbox,NULL);
			}
			gtk_timeout_add(READ_TIMEOUT, read_again, my_options);
		} else {
			gtk_timeout_add(READ_TIMEOUT, wait_again, my_options);
		}
	}
	return(0);
}

gint read_again(gpointer my_options) {
	GString *tmp;
	gboolean again;
	

	tmp = read_fh(((OPTIONS *)my_options)->fh);
	again = TRUE;
	if(tmp) {
		if(tmp->str[tmp->len-1] == 4) {
			again = FALSE;
			tmp = g_string_truncate(tmp, tmp->len-1);
		}
		if(tmp->len) {
			gtk_text_insert (GTK_TEXT(((OPTIONS *)my_options)->msgbox), NULL, NULL, NULL, tmp->str, -1);
			gtk_widget_draw(((OPTIONS *)my_options)->msgbox,NULL);
		}
		g_string_free(tmp,TRUE);
	}
	if(again) {
		gtk_timeout_add(READ_TIMEOUT, read_again, my_options);
	} else {
		close(((OPTIONS *)my_options)->fh);
		gtk_timeout_add(READ_TIMEOUT, wait_again, my_options);
	}
	return(0);
}
