#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <gnome.h>

#include "defs.h"
#include "gui.h"
#include "file.h"
#include "command_line_parser.h"


#ifndef socklen_t
typedef size_t socklen_t;
#endif


/* This function is called after a timeout specified by the user */
gint timeout_func(gpointer my_options) {
	gtk_main_quit();
	return(0);
}

/* gnome-save-session */
static gint save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
              gint is_shutdown, GnomeInteractStyle interact_style,
              gint is_fast, gpointer client_data) {
	return TRUE;
}

/* gnome-session-die */
static gint session_die(GnomeClient* client, gpointer my_options) {
	gtk_main_quit ();
	return TRUE;
}

/* Definitions for popt to get command line arguments */
static char* buttons = NULL;
static char* default_label = NULL;
static char* filename = NULL;
static char* geometry = NULL;
static char* timeout = NULL;
static char* wintitle = NULL;
static char* align_buttons = NULL;

struct poptOption comman_line[] = {
	{ "buttons", 'b', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &buttons, P_BUTTONS, 
			N_("Coma separated buttons list with optionnal integer value"), "label1[:int1],label2[:int2],..." },
	{ "default", 'd', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &default_label, P_DEFAULTS, 
			N_("Defines the default labelled button to be selected when [enter] is pressed"), "label" },
	{ "file", 'f', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &filename, P_FILE, 
			N_("File to display. A filename of '-' reads from the standart input. "
			"A filename of 'socket://hostname:port' reads from the specified host. "
			"A  filename of 'server://port' creates a server that listens on the specified port."), "filename" },
	{ "print", 'p', POPT_ARGFLAG_ONEDASH, NULL, P_PRINT,
			N_("Prints the selected label on the standard output")},
	{ "follow", 'o', POPT_ARGFLAG_ONEDASH, NULL, P_FOLLOW, 
			N_("Reads continuously the input") },
	{ "center", 'c', POPT_ARGFLAG_ONEDASH, NULL, P_CENTER,
			N_("Pop up the window at the center of the screen") },
	{ "nearmouse", 'n', POPT_ARGFLAG_ONEDASH, NULL, P_NEARMOUSE,
			N_("Pop up the window near the mouse cursor")},
	{ "geometry", 'g', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &geometry, P_GEOMETRY, 
			N_("Geometry. See man X for more info.") },
	{ "no_hscrollbar", 0, POPT_ARGFLAG_ONEDASH, NULL, P_NO_HSCROLLBAR, 
			N_("Do not draw an horizontal scrollbar") },
	{ "no_vscrollbar", 0, POPT_ARGFLAG_ONEDASH, NULL, P_NO_VSCROLLBAR, 
			N_("Do not draw an horizontal scrollbar") },
	{ "no_buttons", 0, POPT_ARGFLAG_ONEDASH, NULL, P_NOBUTTONS, 
			N_("Do not draw any button") },
	{ "align_buttons", 0, POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &align_buttons, P_ALIGN_BUTTONS, 
			N_("Align buttons. Possible choices are no, left, center and right."), "no" },
	{ "title", 0, POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &wintitle, P_WINTITLE, 
			N_("Title of the window"), "text" },
	{ "timeout", 't', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &timeout, P_TIMEOUT, 
			N_("Exits after t seconds if the user has not clicked any button yet.") },
	{ "version", 'v', POPT_ARGFLAG_ONEDASH, NULL, P_VERSION,
			N_("Prints the version of " PACKAGE)},
	POPT_AUTOHELP
	{ NULL, 0, 0, NULL, 0 }
};


int main (int argc, char *argv[]) {
	gint return_value;
	OPTIONS *my_options;
	GString *return_label;
	GnomeClient *client;
	poptContext optCon;
	gchar c;
	gchar *tmp;

/* Initializations */
	my_options = (OPTIONS*)g_malloc(sizeof(OPTIONS));
	my_options->message = NULL;
	my_options->value = NULL;
	my_options->label = NULL;
	my_options->number = NULL;
	my_options->filename = NULL;
	my_options->print = FALSE;
	my_options->follow = FALSE;
	my_options->default_label = NULL;
	my_options->clicked_button = 0;
	my_options->win_position = GTK_WIN_POS_NONE;
	my_options->geometry = NULL;
	my_options->timeout = 0;
	my_options->h_scrollbar = TRUE;
	my_options->v_scrollbar = TRUE;
	my_options->nobuttons = FALSE;
	my_options->wintitle = g_string_new(PACKAGE);
	my_options->align_buttons = SB_NO;

/* gnome_init and get arguments */
	bindtextdomain (PACKAGE,GNOMELOCALEDIR);
	textdomain(PACKAGE);
	gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv, comman_line, 0, &optCon);
	optCon = poptGetContext(NULL, argc, argv, comman_line, 0);

/* Processing of arguments */

	if (argc < 2) {
 		poptPrintUsage(optCon, stderr, 0);
		exit(1);
	}

	while ((c = poptGetNextOpt(optCon)) >= 0) {
		switch (c) {
			case P_BUTTONS:
				parse_buttons(g_string_new(buttons), my_options);
				break;
			case P_DEFAULTS:
				my_options->default_label = g_string_new(default_label);
				break;
			case P_FILE:
				my_options->filename = g_string_new(filename);
				break;
			case P_PRINT:
				my_options->print = TRUE;
				break;
			case P_FOLLOW:
				my_options->follow = TRUE;
			case P_CENTER:
				my_options->win_position = GTK_WIN_POS_CENTER;
				break;
			case P_NEARMOUSE:
				my_options->win_position = GTK_WIN_POS_MOUSE;
				break;
			case P_GEOMETRY:
				my_options->geometry = g_string_new(geometry);
				break;
			case P_TIMEOUT:
				my_options->timeout = atoi(timeout)*1000;
				break;
			case P_VERSION : 
				tmp = (gchar*)g_malloc(sizeof(gchar)*MAX_STRING_LENGTH);
				snprintf(tmp, MAX_STRING_LENGTH, 
					_("gmessage+\nVersion %s\nLicence : GPL v2"), VERSION);
				my_options->message = g_string_new(tmp);
				free(tmp);
				break;
			case P_NO_HSCROLLBAR :
				my_options->h_scrollbar = FALSE;
				break;
			case P_NO_VSCROLLBAR :
				my_options->v_scrollbar = FALSE;
				break;
			case P_NOBUTTONS :
				my_options->nobuttons = TRUE;
				break;
			case P_WINTITLE :
				my_options->wintitle = g_string_new(wintitle);
				break;
			case P_ALIGN_BUTTONS :
				if(!strcmp(align_buttons, "no")) my_options->align_buttons = SB_NO;
				else if(!strcmp(align_buttons, "left")) my_options->align_buttons = SB_LEFT;
				else if(!strcmp(align_buttons, "center")) my_options->align_buttons = SB_CENTER;
				else if(!strcmp(align_buttons, "right")) my_options->align_buttons = SB_RIGHT;
				break;
		}
	}

	if (c < -1) {
	/* an error occurred during option processing */
	fprintf(stderr, "%s: %s\n",
		poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
		poptStrerror(c));
		return(-1);
	}
	
	parse_command_line(optCon, my_options);
	    
/* Make the GUI */
	make_gui(my_options);

/* Declare the timeout if necessary */

	if(my_options->timeout) {
		gtk_timeout_add(my_options->timeout, timeout_func, my_options);
	}

/* gnome stuff for gnome session */
  	client = gnome_master_client ();
	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
			GTK_SIGNAL_FUNC (save_session), argv[0]);
	gtk_signal_connect (GTK_OBJECT (client), "die",
			GTK_SIGNAL_FUNC (session_die), my_options);

/* if --follow is enabled, add a timeout function */
	if(my_options->follow) {
		if(my_options->message) {
			if(my_options->message->str[0] != 4)
				gtk_timeout_add(READ_TIMEOUT, read_again, my_options);
		} else gtk_timeout_add(READ_TIMEOUT, read_again, my_options);
	}

/* main loop */

	gtk_main ();

/* close the file if we were if follow mode */
	if(my_options->follow) {
		close(my_options->fh);
	}

/* Return the right argument */

	return_value = g_array_index(my_options->value, gint, my_options->clicked_button);
	if(my_options->print == TRUE) {
		return_label = g_ptr_array_index(my_options->label, my_options->clicked_button);
		g_print("%s\n",return_label->str);
	}

/* clean what we can */
	
	free(my_options);
/* bye */
	return (return_value);
}
