#include <config.h>
#ifdef HAVE_POPT
#include <popt.h>
#include <glib.h>
#include <gettext.h>
#include <libintl.h>
#define _(String) gettext (String)
#    ifdef gettext_noop
#        define N_(String) gettext_noop (String)
#    else
#        define N_(String) (String)
#    endif
#else
#include <gnome.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <signal.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>

#include "netsend.h"

GString *read_fh(gint fh) {
	GString *message;
	gchar tmp[INITIAL_PIPE_BUFFER_LENGTH];
	gint len, total_len;
	
	message = g_string_sized_new(INITIAL_PIPE_BUFFER_LENGTH);
	total_len = 0;
	while((len=read(fh, tmp, INITIAL_PIPE_BUFFER_LENGTH-2)) > 0) {
		tmp[len]=0;
		if(tmp[0] == 4) {
			tmp[1] = 0;
			len = 1;
		}
		total_len += len;
		g_string_append(message, tmp);
	}
	if(total_len) {
		return(message);
	} else {
		g_string_free(message, TRUE);
		return(NULL);
	}
}


int connect_to_socket(GString *ip, uint16_t port) {
	struct hostent server_address, *address;
	struct sockaddr_in *client_sockaddr_in;
	int sock;
	
	if((address = gethostbyname(ip->str)) == NULL) {
		perror("gethostbyname ");
		exit(1);
	}
	memcpy(&server_address,address,sizeof(struct hostent));
	
/*	printf("Local IP address : %d.%d.%d.%d\n",(unsigned char)server_address.h_addr[0], (unsigned char)server_address.h_addr[1], (unsigned char)server_address.h_addr[2], (unsigned char)server_address.h_addr[3]);
 */

/* Open the server socket */

	sock = socket(AF_INET, SOCK_STREAM,6);
	if(sock == -1) {
		perror("Socket ");
		exit(1);
	}
/* Get infos on the server */
	if((client_sockaddr_in = (struct sockaddr_in *)g_malloc(sizeof(struct sockaddr_in)))==NULL) {
		perror("g_malloc on server creation ");
		exit(1);
	}
	
	client_sockaddr_in->sin_family = server_address.h_addrtype;
	client_sockaddr_in->sin_port = htons(port);
	memcpy((char *)&(client_sockaddr_in->sin_addr.s_addr),server_address.h_addr,server_address.h_length);
	bzero((char *)&(client_sockaddr_in->sin_zero),sizeof (struct sockaddr) -
			   				sizeof (uint16_t) -
			   				sizeof (struct in_addr));
	
/* Connect to the server as a client */

	if(connect(sock,client_sockaddr_in, sizeof(struct sockaddr_in))) {
		sock = -1;
	}
	return(sock);
}

static char *mymessage=NULL;
static char *myserver=NULL;
static char *myport=NULL;

struct poptOption command_line[] = {
	{ "message", 'm', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &mymessage, F_MESSAGE, 
			N_("message to send"), "message" },
	{ "server", 's', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &myserver, F_SERVER, 
			N_("name of the server"), "server" },
	{ "port", 'p', POPT_ARG_STRING || POPT_ARGFLAG_ONEDASH, &myport, F_PORT, 
			N_("Port of the server"), "port" },
	POPT_AUTOHELP
	{ NULL, 0, 0, NULL, 0 }
};

#ifdef HAVE_POPT
int main(int argc, const char** argv) {
#else
int main(int argc, char** argv) {
#endif
	GString *message = NULL;
	poptContext optCon;
	gchar c;
	GString *server = NULL;
	char * tmp;
	gint port = -1;
	gint fh;
	gint l;
	
	bindtextdomain (PACKAGE,GNOMELOCALEDIR);
	textdomain(PACKAGE);
#ifndef HAVE_POPT
	gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv, command_line, 0, &optCon);
#endif
	optCon = poptGetContext(NULL, argc, argv, command_line, 0);

/* Processing of arguments */

	if (argc < 7) {
		poptPrintUsage(optCon, stderr, 0);
		exit(1);
	}

	while ((c = poptGetNextOpt(optCon)) >= 0) {
		switch (c) {
			case F_MESSAGE:
				if(!strcmp(mymessage,"-")) {
					/* the message is stdin */
					fh = dup(STDIN_FILENO);
					message = read_fh(fh);
				} else
					message = g_string_new(mymessage);
					message = g_string_append_c(message, 4);
				break;
			case F_SERVER:
				server = g_string_new(myserver);
				break;
			case F_PORT:
				port = atoi(myport);
				if(port <= 0) g_error(_("Wrong port number"));
				break;
		}
	}

	if (c < -1) {
	/* an error occurred during option processing */
		fprintf(stderr, "%s: %s\n",
			poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
			poptStrerror(c));
		return(-1);
	}
	

	fh = connect_to_socket(server, port);
	if(fh == -1) g_error(_("Unable to connect to the specified host."));
	else {
		tmp = (char*)g_malloc(sizeof(char)*INITIAL_PIPE_BUFFER_LENGTH);
		while(!(l = read(fh, tmp, INITIAL_PIPE_BUFFER_LENGTH)));
		free(tmp);
		write(fh, message->str, message->len);
		close(fh);
	}
	exit(0);
}
